package nl.sbdeveloper.mctpaudio.listener;

import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.UnsupportedTagException;
import com.sk89q.worldguard.protection.regions.ProtectedRegion;
import net.raidstone.wgevents.events.RegionEnteredEvent;
import net.raidstone.wgevents.events.RegionLeftEvent;
import nl.sbdeveloper.mctpaudio.ThemeParkAudio;
import nl.sbdeveloper.mctpaudio.api.AudioType;
import nl.sbdeveloper.mctpaudio.api.events.AudioConnectionUpdateEvent;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import nl.sbdeveloper.mctpaudio.socket.messages.AudioMessage;
import nl.sbdeveloper.mctpaudio.utils.HeadUtil;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.scheduler.BukkitRunnable;

import java.io.IOException;
import java.util.List;
import java.util.UUID;

/**
 * This class handles playing numbers in a region.
 * This will NOT be used for region much, only if you start playing music in a region with the @a[region=...] selector.
 */
public class PlayInRegionHandler implements Listener {
    private final String region;
    private final String url;

    private final List<UUID> players;
    private int currentTick = 0;
    private boolean isListening = true;

    public PlayInRegionHandler(String region, String url, List<UUID> players) {
        this.region = region;
        this.url = url;
        this.players = players;

        load();
        Bukkit.getPluginManager().registerEvents(this, ThemeParkAudio.getPlugin());
    }

    /**
     * Sends the song to every audio client, and starts it.
     */
    private void load() {
        int ticks;
        try {
            ticks = HeadUtil.getTicksOfFile(url);
        } catch (IOException | InvalidDataException | UnsupportedTagException e) {
            e.printStackTrace();
            return;
        }

        new BukkitRunnable() {
            @Override
            public void run() {
                currentTick++;
                if (currentTick == ticks) {
                    isListening = false;
                    cancel();
                }
            }
        }.runTaskTimer(ThemeParkAudio.getPlugin(), 0L, 1L);

        AudioMessage.of(AudioType.SFX, url, true).send(players, PinManager::hasPin);
    }

    private void start(UUID pUUID) {
        start(pUUID, 0);
    }

    /**
     * Starts the song for a Player.
     * @param pUUID The player to start it for.
     */
    private void start(UUID pUUID, int tick) {
        if (!isListening) return;

        Player p = Bukkit.getPlayer(pUUID);
        if (p == null) return;
        if (!PinManager.hasPin(p.getUniqueId())) return;

        AudioMessage.of(AudioType.SFX, url, true, tick * 50).send(pUUID);
    }

    /**
     * Stops the song for a Player.
     * @param pUUID The player to stop it for.
     */
    private void stop(UUID pUUID) {
        if (!isListening) return;

        Player p = Bukkit.getPlayer(pUUID);
        if (p == null) return;
        if (!PinManager.hasPin(p.getUniqueId())) return;

        AudioMessage.of(AudioType.SFX, url, false).send(pUUID);
    }

    @EventHandler
    private void onEnterRegion(RegionEnteredEvent e) {
        if (e.getPlayer() == null) return;

        if (e.getRegionName().equals(region)) {
            if (!players.contains(e.getPlayer().getUniqueId())) {
                players.add(e.getPlayer().getUniqueId());
                start(e.getPlayer().getUniqueId(), currentTick);
            } else {
                start(e.getPlayer().getUniqueId());
            }
        }
    }

    @EventHandler
    private void onLeaveRegion(RegionLeftEvent e) {
        if (e.getPlayer() == null) return;

        if (e.getRegionName().equals(region)) {
            if (!players.contains(e.getPlayer().getUniqueId())) return;
            stop(e.getPlayer().getUniqueId());
        }
    }

    @EventHandler
    private void onConnectionUpdate(AudioConnectionUpdateEvent e) {
        if (e.isConnected()) {
            List<ProtectedRegion> regionsIn = WGManager.getRegionsIn(e.getPlayer().getLocation());
            if (regionsIn.stream().anyMatch(reg -> reg.getId().equals(region))) {
                players.add(e.getPlayer().getUniqueId());
                e.setCancelled(true);
                start(e.getPlayer().getUniqueId(), currentTick);
            }
        } else {
            if (!players.contains(e.getPlayer().getUniqueId())) return;
            stop(e.getPlayer().getUniqueId());
            players.remove(e.getPlayer().getUniqueId());
        }
    }
}
