package nl.sbdeveloper.mctpaudio.radio;

import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.UnsupportedTagException;
import lombok.Getter;
import nl.sbdeveloper.mctpaudio.ThemeParkAudio;
import nl.sbdeveloper.mctpaudio.api.AudioType;
import nl.sbdeveloper.mctpaudio.api.maps.SongList;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.socket.messages.AudioMessage;
import nl.sbdeveloper.mctpaudio.utils.HeadUtil;
import nl.sbdeveloper.mctpaudio.utils.DiscordUtil;
import nl.sbdeveloper.mctpaudio.utils.WebhookMessage;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.awt.*;
import java.io.IOException;

public class Playlist {
    private final SongList<String> playList = new SongList<>();

    @Getter
    private boolean running = false;
    private BukkitTask currentTimer;
    private int faultCounter = 0;

    /**
     * Create a new PlayList object
     * Starts the playlist
     */
    public Playlist() {
        start();
    }

    /**
     * Start this playlist
     */
    public void start() {
        for (String URL : ThemeParkAudio.getPlugin().getConfig().getStringList("RadioSongs")) {
            addSong(URL);
        }

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkAudio.getPlugin(), this::nextSong);

        running = true;
    }

    /**
     * Stop the playlist
     */
    public void stop() {
        playList.clear();

        if (currentTimer != null) {
            currentTimer.cancel();
            currentTimer = null;
        }

        running = false;
    }

    /**
     * Add a song by the url
     *
     * @param url The song url (mp3)
     */
    public void addSong(String url) {
        playList.add(url);
        playList.shuffle();
    }

    /**
     * Go to the next song
     */
    public void nextSong() {
        if (currentTimer != null) return; //A song is playing?

        if (faultCounter > 4) {
            ThemeParkAudio.getPlugin(ThemeParkAudio.class).getLogger().severe("Detected to many IO errors! Stopping the radio...");
            stop(); //FALLBACK! 4 errors occured.
            return;
        }

        if (playList.isEmpty()) {
            start();
            return;
        }

        //Get song
        String nextURL = playList.getRandom();
        if (nextURL == null) return;

        //Remove from PlayList
        try {
            playList.remove(nextURL);
        } catch (ArrayIndexOutOfBoundsException ignored) {}

        //Get ticks of song
        int ticks;
        try {
            ticks = HeadUtil.getTicksOfFile(nextURL);
        } catch (InvalidDataException | UnsupportedTagException ex) {
            WebhookMessage.EmbedObject embed = new WebhookMessage.EmbedObject();
            embed.setTitle("Radio song failure!");
            embed.setColor(Color.RED);
            embed.setDescription("A song in the radio does not have MPEG headers. A song without headers cannot be played in the radio because the length cannot be determined. Please fix as soon as possible.\n\nClick on the title to go to the song.");
            embed.setUrl(nextURL);
            try {
                DiscordUtil.sendMessageDiscord(embed);
            } catch (IOException e) {
                e.printStackTrace();
            }
            nextSong();
            return;
        } catch (IOException ex) {
            faultCounter++;
            nextSong();
            return;
        }

        if (ticks <= 0) {
            WebhookMessage.EmbedObject embed = new WebhookMessage.EmbedObject();
            embed.setTitle("Radio song failure!");
            embed.setColor(Color.RED);
            embed.setDescription("A song in the radio does not have MPEG headers. A song without headers cannot be played in the radio because the length cannot be determined. Please fix as soon as possible.\n\nClick on the title to go to the song.");
            embed.setUrl(nextURL);
            try {
                DiscordUtil.sendMessageDiscord(embed);
            } catch (IOException e) {
                e.printStackTrace();
            }
            nextSong();
            return;
        }

        AudioMessage.of(AudioType.RADIO, nextURL).broadcast(PinManager::hasPin);

        Bukkit.getLogger().info("Started song with duration: " + ticks / 20 + " sec.");

        faultCounter = 0;

        //And started timer, so that it starts a new song if the old one is done
        currentTimer = Bukkit.getScheduler().runTaskLaterAsynchronously(ThemeParkAudio.getPlugin(), () -> {
            currentTimer = null;
            nextSong();
        }, ticks);

        //And shuffle the playlist now
        playList.shuffle();
    }
}
